@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use vars qw( $VERSION @EXPORT @EXPORT_OK @ISA $CurrentPackage @IncludeLibs $ScanFileRE );
 
-$VERSION   = '1.26';
+$VERSION   = '1.28';
 @EXPORT    = qw( scan_deps scan_deps_runtime );
 @EXPORT_OK = qw( scan_line scan_chunk add_deps scan_deps_runtime path_to_inc_name );
 
@@ -802,6 +802,9 @@ sub scan_line {
   CHUNK:
     foreach (split(/;/, $line)) {
         s/^\s*//;
+        #  handle single line blocks like 'do { package foo; use xx; }'
+        s/^(?:do\s*)?\{\s*//;  
+        s/\}$//;
 
         if (/^package\s+(\w+)/) {
             $CurrentPackage = $1;
@@ -915,6 +918,9 @@ sub scan_chunk {
     my $module = eval {
         $_ = $chunk;
         s/^\s*//;
+        #  handle single line blocks like 'do { package foo; use xx; }'
+        s/^(?:do\s*)?\{\s*//;  
+        s/\}\s*$//;
 
         # TODO: There's many more of these "loader" type modules on CPAN!
         # scan for the typical module-loader modules
@@ -1018,10 +1024,10 @@ sub scan_chunk {
 }
 
 sub _find_encoding {
+    my ($enc) = @_;
     return unless $] >= 5.008 and eval { require Encode; %Encode::ExtModule };
 
-    my $mod = $Encode::ExtModule{ Encode::find_encoding($_[0])->name }
-      or return;
+    my $mod = eval { $Encode::ExtModule{ Encode::find_encoding($enc)->name } } or return;
     $mod =~ s{::}{/}g;
     return "$mod.pm";
 }
@@ -1100,22 +1106,19 @@ sub add_deps {
             next;
         }
 
-        my $type = _gettype($file);
         _add_info( rv     => $rv,   module  => $module,
                    file   => $file, used_by => $used_by,
-                   type   => $type );
+                   type   => _gettype($file) );
 
-        if ($module =~ /(.*?([^\/]*))\.p[mh]$/i) {
-            my ($path, $basename) = ($1, $2);
+        if ((my $path = $module) =~ s/\.p[mh]$//i) {
 
             foreach (_glob_in_inc("auto/$path")) {
-                next if $_->{file} =~ m{\bauto/$path/.*/};  # weed out subdirs
-                next if $_->{name} =~ m{/(?:\.exists|\.packlist|\Q$Config{lib_ext}\E)$};
-                $type = _gettype($_->{name});
+                next if $_->{name} =~ m{^auto/$path/.*/};  # weed out subdirs
+                next if $_->{name} =~ m{/(?:\.exists|\.packlist)$|\Q$Config{lib_ext}\E$};
 
                 _add_info( rv     => $rv,        module  => $_->{name},
                            file   => $_->{file}, used_by => $module,
-                           type   => $type );
+                           type   => _gettype($_->{name}) );
             }
 
             ### Now, handle module and distribution share dirs
@@ -1196,7 +1199,7 @@ sub _glob_in_inc_1 {
         my $dir = "$inc/$subdir";
         next unless -d $dir;
 
-        opendir my $dh, $dir or next; 
+        opendir(my $dh, $dir) or next; 
         my @names = map { "$subdir/$_" } grep { -f "$dir/$_" } readdir $dh;
         closedir $dh;
 
@@ -1346,18 +1349,20 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         $execute ? "END\n" : "CHECK\n", 
         <<'...';
 {
-    # save %INC etc so that requires below don't pollute them
-    my %_INC = %INC;
-    my @_INC = @INC;
+    require DynaLoader;
     my @_dl_shared_objects = @DynaLoader::dl_shared_objects;
     my @_dl_modules = @DynaLoader::dl_modules;
 
+    # save %INC etc so that requires below don't pollute them
+    my %_INC = %INC;
+    my @_INC = @INC;
+
     require Cwd;
-    require DynaLoader;
     require Data::Dumper;
     require Config;
+    my $dlext = $Config::Config{dlext};
 
-    while (my ($k, $v) = each %_INC)
+    foreach my $k (keys %_INC)
     {
         # NOTES:
         # (1) An unsuccessful "require" may store an undefined value into %INC.
@@ -1368,6 +1373,8 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         #     "Class/MOP/Class/Immutable/Moose/Meta/Class.pm" => "(set by Moose)"
         #     On some architectures (e.g. Windows) Cwd::abs_path() will throw
         #     an exception for such a pathname.
+
+        my $v = $_INC{$k};
         if (defined $v && !ref $v && -e $v)
         {
             $_INC{$k} = Cwd::abs_path($v);
@@ -1381,10 +1388,9 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
     # drop refs from @_INC
     @_INC = grep { !ref $_ } @_INC;
 
-    my $dlext = $Config{dlext};
     my @dlls = grep { defined $_ && -e $_ } Module::ScanDeps::DataFeed::_dl_shared_objects();
     my @shared_objects = @dlls; 
-    push @shared_objects, grep { s/\Q.$dlext\E$/\.bs/ && -e $_ } @dlls;
+    push @shared_objects, grep { s/\.\Q$dlext\E$/.bs/ && -e $_ } @dlls;
 
     # write data file
     my $data_file = $ENV{MSD_DATA_FILE};
@@ -1421,8 +1427,8 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         my $modpname = join('/', @modparts);
 
         foreach my $dir (@_INC) {
-            my $file = "$dir/auto/$modpname/$modfname.$Config{dlext}";
-            return $file if -r $file;
+            my $file = "$dir/auto/$modpname/$modfname.$dlext";
+            return $file if -e $file;
         }
         return;
     }
@@ -1469,8 +1475,8 @@ sub _info2rv {
 
     require File::Spec;
 
-    while (my ($key, $path) = each %{ $info->{'%INC'} }) {
-        $path =~ s:\\:/:g;
+    foreach my $key (keys %{ $info->{'%INC'} }) {
+        (my $path = $info->{'%INC'}{$key}) =~ s:\\:/:g;
         $key =~ s:\\:/:g;
         $key =~ s/$strip_inc_prefix// if File::Spec->file_name_is_absolute($key);
 
